/*
 * hid.c
 *
 * The hid functions
 *
 * Copyright (C) 2001 Greg Kroah-Hartman <greg@kroah.com>
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation version 2 of the License.
 * 
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *	General Public License for more details.
 * 
 *	You should have received a copy of the GNU General Public License along
 *	with this program; if not, write to the Free Software Foundation, Inc.,
 *	675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stddef.h>	/* for NULL */
#include <stdlib.h>	/* for getenv() */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include "hotplug.h"
#include "System2AP.h"


#ifndef RESCUE_LINUX
// devpath = "/devices/platform/ohci/usb2/2-2/2-2:1.0/0003:046D:C309.0001"
// or        "/devices/platform/ehci/usb1/1-2/1-2.4/1-2.4:1.0/0003:046D:C309.0003"
static int get_hid_locate(char *locate)
{
	char *devpath, *ptr, *ptr1;

	devpath = getenv ("DEVPATH");

	// ptr = "usb1/1-2/1-2.4/1-2.4:1.0/0003:046D:C309.0003"
	ptr = strstr(devpath, "usb");
	if(ptr == NULL) // it may be a SATA device
		return 1;
	
	// ptr1 should not be NULL
	ptr1 = strstr(ptr, ":");
	if(ptr1)
		*ptr1 = '\0';
	else
		return 1;

	// ptr = "usb1/1-2/1-2.4/1-2.4"
	while((ptr1 = strstr(ptr, "/")))
		ptr = ptr1 + 1;

	sprintf(locate, "%s", ptr);
	return 0;
}
#endif

static int hid_add (void)
{
	return 0;
}

static int hid_remove (void)
{
	return 0;
}

static int usb_kbd_up (void) {
	char ptr[128];
	SYS2AP_MESSAGE sysmsg;
	printf("#@# cfyeh-debug %s(%d)\n", __func__, __LINE__);

#ifndef RESCUE_LINUX
	if(get_hid_locate(ptr) == 0)
	{
// +++ cyhuang (2011/10/03) :
    char *evdev = getenv ("EVENT");
// +++ cyhuang

	printf("Hotplug got one USB Hotplug of \"KBD UP\" from port \"%s\" (event=%s).\n", ptr,evdev);
	sysmsg.m_type = SYS2AP_MSG_USBHID_STATUS;
// +++ cyhuang (2011/10/03) :
    sprintf(sysmsg.m_msg, "%s %s KBD %s", ptr, SYS2AP_MSG_UP,evdev);
// +++ cyhuang
	RTK_SYS2AP_SendMsg(&sysmsg);
	}
#endif
	return 0;
}

static int usb_kbd_down (void) {
	char ptr[128];
	SYS2AP_MESSAGE sysmsg;
	printf("#@# cfyeh-debug %s(%d)\n", __func__, __LINE__);

#ifndef RESCUE_LINUX
	if(get_hid_locate(ptr) == 0)
	{
// +++ cyhuang (2011/10/03) :
    char *evdev = getenv ("EVENT");
// +++ cyhuang
	printf("Hotplug got one USB Hotplug of \"KBD DOWN\" from port \"%s\" (event=%s).\n", ptr,evdev);
	sysmsg.m_type = SYS2AP_MSG_USBHID_STATUS;
// +++ cyhuang (2011/10/03) :
    sprintf(sysmsg.m_msg, "%s %s KBD %s", ptr, SYS2AP_MSG_DOWN,evdev);
// +++ cyhuang
	RTK_SYS2AP_SendMsg(&sysmsg);
	}
#endif
	return 0;
}

static int usb_mice_up (void) {
	char ptr[128];
	SYS2AP_MESSAGE sysmsg;
	printf("#@# cfyeh-debug %s(%d)\n", __func__, __LINE__);

#ifndef RESCUE_LINUX
	if(get_hid_locate(ptr) == 0)
	{
// +++ cyhuang (2011/10/03) :
    char *evdev = getenv ("EVENT");
// +++ cyhuang
	printf("Hotplug got one USB Hotplug of \"MICE UP\" from port \"%s\" (event=%s).\n", ptr,evdev);
	sysmsg.m_type = SYS2AP_MSG_USBHID_STATUS;
	sprintf(sysmsg.m_msg, "%s %s MICE %s", ptr, SYS2AP_MSG_UP,evdev);
	RTK_SYS2AP_SendMsg(&sysmsg);
	}
#endif
	return 0;
}

static int usb_mice_down (void) {
	char ptr[128];
	SYS2AP_MESSAGE sysmsg;

#ifndef RESCUE_LINUX
	if(get_hid_locate(ptr) == 0)
	{
// +++ cyhuang (2011/10/03) :
    char *evdev = getenv ("EVENT");
// +++ cyhuang
	printf("Hotplug got one USB Hotplug of \"MICE DOWN\" from port \"%s\" (event=%s).\n", ptr,evdev);
	sysmsg.m_type = SYS2AP_MSG_USBHID_STATUS;
	sprintf(sysmsg.m_msg, "%s %s MICE %s", ptr, SYS2AP_MSG_DOWN,evdev);
	RTK_SYS2AP_SendMsg(&sysmsg);
	}
#endif
	return 0;
}

static int usb_mt_up (void) {
	char ptr[128];
	SYS2AP_MESSAGE sysmsg;
	printf("#@# cfyeh-debug %s(%d)\n", __func__, __LINE__);

#ifndef RESCUE_LINUX
	if(get_hid_locate(ptr) == 0)
	{
// +++ cyhuang (2011/10/03) :
    char *evdev = getenv ("EVENT");
// +++ cyhuang
	printf("Hotplug got one USB Hotplug of \"MT UP\" from port \"%s\" (event=%s).\n", ptr,evdev);
	sysmsg.m_type = SYS2AP_MSG_USBHID_STATUS;
	sprintf(sysmsg.m_msg, "%s %s MT %s", ptr, SYS2AP_MSG_UP,evdev);
	RTK_SYS2AP_SendMsg(&sysmsg);
	}
#endif
	return 0;
}

static int usb_mt_down (void) {
	char ptr[128];
	SYS2AP_MESSAGE sysmsg;

#ifndef RESCUE_LINUX
	if(get_hid_locate(ptr) == 0)
	{
// +++ cyhuang (2011/10/03) :
    char *evdev = getenv ("EVENT");
// +++ cyhuang
	printf("Hotplug got one USB Hotplug of \"MT DOWN\" from port \"%s\" (event=%s).\n", ptr,evdev);
	sysmsg.m_type = SYS2AP_MSG_USBHID_STATUS;
	sprintf(sysmsg.m_msg, "%s %s MT %s", ptr, SYS2AP_MSG_DOWN,evdev);
	RTK_SYS2AP_SendMsg(&sysmsg);
	}
#endif
	return 0;
}

static int usb_joystick_up (void) {
	char ptr[128];
	SYS2AP_MESSAGE sysmsg;
	printf("#@# cfyeh-debug %s(%d)\n", __func__, __LINE__);

#ifndef RESCUE_LINUX
	if(get_hid_locate(ptr) == 0)
	{
// +++ cyhuang (2011/10/03) :
    char *evdev = getenv ("EVENT");
// +++ cyhuang
	printf("Hotplug got one USB Hotplug of \"JOYSTICK UP\" from port \"%s\" (event=%s).\n", ptr,evdev);
	sysmsg.m_type = SYS2AP_MSG_USBHID_STATUS;
	sprintf(sysmsg.m_msg, "%s %s JOYSTICK %s", ptr, SYS2AP_MSG_UP,evdev);
	RTK_SYS2AP_SendMsg(&sysmsg);
	}
#endif
	return 0;
}

static int usb_joystick_down (void) {
	char ptr[128];
	SYS2AP_MESSAGE sysmsg;

#ifndef RESCUE_LINUX
	if(get_hid_locate(ptr) == 0)
	{
// +++ cyhuang (2011/10/03) :
    char *evdev = getenv ("EVENT");
// +++ cyhuang
	printf("Hotplug got one USB Hotplug of \"JOYSTICK DOWN\" from port \"%s\" (event=%s).\n", ptr,evdev);
	sysmsg.m_type = SYS2AP_MSG_USBHID_STATUS;
	sprintf(sysmsg.m_msg, "%s %s JOYSTICK %s", ptr, SYS2AP_MSG_DOWN,evdev);
	RTK_SYS2AP_SendMsg(&sysmsg);
	}
#endif
	return 0;
}

static struct subsystem hid_subsystem[] = {
	{ ADD_STRING, hid_add },
	{ REMOVE_STRING, hid_remove },
	{ USB_KBD_UP, usb_kbd_up },
	{ USB_KBD_DOWN, usb_kbd_down },
	{ USB_MICE_UP, usb_mice_up },
	{ USB_MICE_DOWN, usb_mice_down },
	{ USB_MT_UP, usb_mt_up },
	{ USB_MT_DOWN, usb_mt_down },
	{ USB_JOYSTICK_UP, usb_joystick_up },
	{ USB_JOYSTICK_DOWN, usb_joystick_down },
	{ NULL, NULL }
};

int hid_handler (void)
{
	char * action;
	
	action = getenv ("ACTION");
	dbg ("action = %s\n", action);
	if (action == NULL) {
		dbg ("missing ACTION environment variable, aborting.");
		return 1;
	}

	return call_subsystem (action, hid_subsystem);
}
